/* NiuTrans.Tensor - an open-source tensor library
* Copyright (C) 2017, Natural Language Processing Lab, Northestern University.
* All rights reserved.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*   http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

/*
* $Created by: JIANG Yufan (email: jiangyufan2018@outlook.com) 2019-04-05
*/

#include <math.h>
#include "../../XName.h"
#include "Binary.h"
#include "Binary.cuh"

namespace nts {

int scale(int x, int scale)
{
    return x * scale;
}

int descale(int x, int descale)
{
    return x / descale;
}

int shift(int x, int shift)
{
    return x + shift;
}

int mod(int x, int mod)
{
    return x % mod;
}

#ifdef USE_CUDA
/* define three marco separately, specify the respective function names  (GPU mode) */
#define _SIMPLE_BINARY_FUNCTION(_funcName, _cudaFuncName, origFunc)         \
void _funcName(const XTensor * a, XTensor * b, int num)                     \
{                                                                           \
    /* run it on GPUs */                                                    \
    if (a->devID >= 0) {                                                    \
        _cudaFuncName(a, b, num);                                           \
        b->Dump(stderr, "zxc");                                             \
        return;                                                             \
    }                                                                       \
    CheckNTErrors((XTensor::IsSameShaped(a, b)),                            \
                "Input tensors should have the same type!");                \
    CheckNTErrors((a->dataType == X_INT), "TODO!");                         \
    int * d = (int*)a->data;                                                \
    int * db = (int*)b->data;                                               \
    for (int i = 0; i < a->unitNum; i++)                                    \
        db[i] = (int)origFunc(d[i], num);                                   \
}

#define SIMPLE_BINARY_FUNCTION(funcName, _funcName)                         \
XTensor funcName(const XTensor &a, int num)                                 \
{                                                                           \
    XTensor b(&a);                                                          \
    b.SetTMPFlag();                                                         \
    _funcName(&a, &b, num);                                                 \
    b.Dump(stderr, "asd");                                                  \
    return b;                                                               \
}

_SIMPLE_BINARY_FUNCTION(_Scale, _CudaScale, scale)
SIMPLE_BINARY_FUNCTION(Scale, _Scale)

_SIMPLE_BINARY_FUNCTION(_DeScale, _CudaDeScale, descale)
SIMPLE_BINARY_FUNCTION(DeScale, _DeScale)

_SIMPLE_BINARY_FUNCTION(_Shift, _CudaShift, shift)
SIMPLE_BINARY_FUNCTION(Shift, _Shift)

_SIMPLE_BINARY_FUNCTION(_Mod, _CudaMod, mod)
SIMPLE_BINARY_FUNCTION(Mod, _Mod)

#else
/* define three marco separately, specify the respective function names (CPU mode) */
#define _SIMPLE_BINARY_FUNCTION(_funcName, _cudaFuncName, origFunc)         \
void _funcName(const XTensor * a, XTensor * b, int num)                     \
{                                                                           \
    /* run it on GPUs */                                                    \
    if (a->devID >= 0) {                                                    \
        _cudaFuncName(a, b, num);                                           \
        return;                                                             \
    }                                                                       \
    CheckNTErrors((XTensor::IsSameShaped(a, b)),                            \
                "Input tensors should have the same type!");                \
    CheckNTErrors((a->dataType == DEFAULT_DTYPE), "TODO!");                 \
    int * d = (int*)a->data;                                                \
    int * db = (int*)b->data;                                               \
    for (int i = 0; i < a->unitNum; i++)                                    \
        db[i] = (int)origFunc(d[i], num);                                   \
}

#define SIMPLE_BINARY_FUNCTION(funcName, _funcName)                         \
void funcName(const XTensor & a, XTensor &b, int num)                       \
{                                                                           \
    _funcName(&a, &b, num);                                                 \
}

_SIMPLE_BINARY_FUNCTION(_Scale, _CudaScale, scale)
SIMPLE_BINARY_FUNCTION(Scale, _Scale)

_SIMPLE_BINARY_FUNCTION(_DeScale, _CudaDeScale, descale)
SIMPLE_BINARY_FUNCTION(DeScale, _DeScale)

_SIMPLE_BINARY_FUNCTION(_Shift, _CudaShift, shift)
SIMPLE_BINARY_FUNCTION(Shift, _Shift)

_SIMPLE_BINARY_FUNCTION(_Mod, _CudaMod, mod)
SIMPLE_BINARY_FUNCTION(Mod, _Mod)

#endif

} // namespace nts(NiuTrans.Tensor)